$.ajaxSetup({
  beforeSend: function(xhr, settings) {
    function getCookie(name) {
      var cookieValue = null;
      if (document.cookie && document.cookie !== '') {
        var cookies = document.cookie.split(';');
        for (var i = 0; i < cookies.length; i++) {
          var cookie = jQuery.trim(cookies[i]);
          // Does this cookie string begin with the name we want?
          if (cookie.substring(0, name.length + 1) == (name + '=')) {
            cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
            break;
          }
        }
      }
      return cookieValue;
    }
    if (! (/^http:.*/.test(settings.url) || /^https:.*/.test(settings.url))) {
      // Only send the token to relative URLs i.e. locally.
      xhr.setRequestHeader("X-CSRFToken", getCookie('csrftoken'));
    }
  }
});

var mistakes = {
  'url' : '/ajax/add_mistake/',
  'maxLength': 256,
  'contextLength': 70,
  'formContent' : void 0
};

var mistakesBlockSelector = '[data-mistake="content"]',
    mistakesBlock;

var testSelection = function(selection) {
  if (window.getSelection || document.getSelection) {
    var start = $(selection.anchorNode).closest(mistakesBlockSelector),
        end   = $(selection.focusNode).closest(mistakesBlockSelector);
    return (start.length && end.length);
  } else {
    var range_content = selection.createRange();
    range_content.moveToElementText(mistakesBlock);

    return range_content.inRange(selection.createRange());
  }
};

var cleaning = function(parent) {
  $.each(parent.childNodes, function(i,node) {
    if(node.nodeName.toLowerCase() == 'script' ||
        node.nodeName.toLowerCase() == 'ymaps' ||
        node.nodeName.toLowerCase() == 'noscript' ||
        (node.nodeName.toLowerCase() == 'div' &&
            $.inArray('esir.ignore', node.classList) >= 0)
        ){

      node.textContent = "";
      return;
    }
    if (node.childNodes.length) {
      cleaning(node);
    }
  })
};

var cleaningRange = function(range) {

  var fragment = range.cloneContents();

  cleaning(fragment);

  var rng = document.createRange();
  rng.selectNodeContents(fragment);
  return rng;
};

var getSelect = function() {
  var text,
      range = null,
      range_pre = null,
      range_suf = null;


  var selection = null;
  if (window.getSelection) {
    selection = window.getSelection();
  } else {
    if (document.getSelection) {
      selection = document.getSelection();
    } else {
      selection = document.selection;
    }
  }

  if (!testSelection(selection)) {return false;}

  if (selection != null) {
    var pre = "", suf = "";
    if (selection.getRangeAt) {
      range = selection.getRangeAt(0);
      text = range.toString();
      range_pre = document.createRange();
      range_pre.setStartBefore(mistakesBlock);
      range_pre.setEnd(range.startContainer, range.startOffset);
      pre = cleaningRange(range_pre).toString();
      range_suf = range.cloneRange();
      range_suf.setStart(range.endContainer, range.endOffset);
      range_suf.setEndAfter(mistakesBlock);

      suf = cleaningRange(range_suf).toString();
    } else {
      if (selection.createRange) {
        range = selection.createRange();
        text = range.text;
        range_pre = selection.createRange();
        range_pre.moveStart("character", -mistakes.contextLength);
        range_pre.moveEnd("character", -text.length);
        pre = range_pre.text;
        range_suf = selection.createRange();
        range_suf.moveEnd("character", mistakes.contextLength);
        range_suf.moveStart("character", text.length);
        suf = range_suf.text;
      } else {
        text = "" + selection;
      }
    }
    var p;
    var s = (p = text.match(/^(\s*)/)) && p[0].length;
    var e = (p = text.match(/(\s*)$/)) && p[0].length;
    pre = (pre + text.substring(0, s)).replace(/\s+/gm, ' ');
    suf = (text.substring(text.length - e, text.length) + suf).replace(/\s+/gm, ' ');
    text = text.substring(s, text.length - e);

    pre = pre.substring(pre.length - mistakes.contextLength, pre.length).replace(/^\S{1,10}\s+/, "");
    suf = suf.substring(0, mistakes.contextLength).replace(/\s+\S{1,10}$/, "");

    if (text == "") {
      return false;
    }

    var datatext = {
      "pre":  pre,
      "text": text,
      "suf":  suf
    };
  } else {
    alert('Ваш браузер не поддерживает работу с выделенного текста. Возможно, слишком старая версия.');
    return false;
  }
  return datatext;
};

var send = function(datatext, defer) {
  $.ajax({
    url : mistakes.url,
    type: 'POST',
    data: datatext,
    success: function(data) {
      if (data.success) {
        defer.resolve();
      } else {
        defer.reject(data);
      }
    }
  });
};

var showErrors = function(form, errors) {
  $('.error', form).remove();
  $.each(errors, function(key, error) {
    var err = $();
    $.each(error, function(i, value) {
      err = err.add('<span class="error">'+value+'</span>');
    });
    if (key == "__all__") {
      err.prependTo(form)
    } else {
      $('[name="'+key+'"]', form).after(err);
    }
  })
};

var bindSymbolCounter = function(textareas) {
  if (textareas.length <= 0) {
    return false;
  }

  var getLength = function(value) {
    var value_spaces = value.match(/\n/gm);

    return value.length + ((!value_spaces) ? 0 : value_spaces.length);;
  };

  textareas.each(function(i, item) {
    var $item = $(item)
    var counter = $('<span class="message_counter">(0/256)</span>');
    $item.data('counter', counter);
    $item.before(counter);
  });

  textareas.bind('change', function() {
    var message = $(this),
        message_value = message.val(),
        message_length = getLength(message_value);

    var max = 256;

    if(message_length > max){

      var tmp_msg = message_value;
      tmp_msg = tmp_msg.replace(/\n/gm, '^^');
      tmp_msg = tmp_msg.substr(0, max + 1);

      if (tmp_msg.lastIndexOf('^^') == max -1) {
        tmp_msg = tmp_msg.replace(/\^+$/gm, '');
      } else {
        tmp_msg = tmp_msg.substr(0, max);
      }

      tmp_msg = tmp_msg.replace(/\^{2}/gm, '\n');
      message_value = tmp_msg;
      message.val(tmp_msg);
      message.scrollTop(5000);
    }

    message_length = getLength(message_value);
    message.data('counter').text('(' + message_length + '/' + max + ')');
  });

  var mesTimer;
  textareas.bind("keyup", function(){ $(this).trigger('change') });

  textareas.bind("blur", function(){
    clearInterval(mesTimer);
  });

  textareas.bind("focus", function(){
    var field = $(this);
    mesTimer = setInterval(function(){
      field.trigger('change');
    }, 300);
  });

};

var showDialog = function(datatext) {
  if (datatext.text.length > mistakes.maxLength) {
    alert('Длина выделенного текста не должна быть больше '+ mistakes.maxLength + ' символов.\n\nВыделено: ' + datatext.text.length);
    return;
  }

  var data = {
    'url' : window.location.href,
    'text' : datatext.text,
    'before_text' : datatext.pre,
    'after_text' : datatext.suf,
    'comment' : ''
  };

  var form = mistakes.formContent.clone();
  form.dialog({
    modal: true,
    width: 500,
    title : 'Запрос на актуализацию контента',
    buttons: {
      'Отправить': function() {
        var $el = $(this),
            defer = $.Deferred();

        var form = $('form', this);
        defer.done(function() {
          form.html('<br /><br /><br /><p>Спасибо! Ваш запрос успешно отправлен</p>');
          $el.dialog( {
            buttons: {},
            title: 'Запрос успешно отправлен'
          } );
//          $el.dialog( "close" );
        }).fail(function(data) {
          showErrors(form, data.errors);
        });

        send(form.serialize(), defer);
      },
      'Отменить': function() {
        $(this).dialog( "close" );
      }
    },
    show: function() {
      var $el = $(this);

      $('[data-mistake="text"]').html('' +
          data.before_text + '' +
          '<strong>' +
            data.text +
          '</strong>' +
          data.after_text
      );

      $.each(data, function(key, value) {
        $('[name="'+ key +'"]', $el).val(value);
      });

      bindSymbolCounter($('textarea', $el));
    },
    close: function() {
      $(this).dialog( "destroy" ).remove();
    }
  });

};
var errorDetected = function(datatext) {
  if (!mistakes.formContent) {
    $.ajax({
      'type': 'GET',
      'url' : mistakes.url,
      'success' : function(data) {
        mistakes.formContent = $(data);
        showDialog(datatext);
      }
    });
  } else {
    showDialog(datatext);
  }
};

var initMistakes = function() {
  mistakesBlock = $(mistakesBlockSelector)[0];
  $('body').on('keyup', function(e) {
    if (e.ctrlKey && e.keyCode == 13) {
      var select = getSelect();
      if (select) {
        errorDetected(select);
      }
    }
  });
};


$(function(){
  initMistakes();
});